import {NgModule} from "@angular/core";
import {RouterModule, Routes} from "@angular/router";
import {LoginPresentational} from "@app/components/presentationals/login/login.presentational";
import {PageNotFoundPresentational} from "@app/components/presentationals/page-not-found/page-not-found.presentational";
import {AuthGuardService} from "@app/shared/guards/auth-guard.service";
import {AppRoutesEnum} from "@app/shared/enums/routes.enum";
import {TRANSLATE} from "solidify-frontend";


const routes: Routes = [
  {
    path: AppRoutesEnum.access,
    loadChildren: "./features/home/home.module#HomeModule",
    data: {
      breadcrumb: TRANSLATE("breadcrumb.access.root"),
    },
  },
  {
    path: AppRoutesEnum.deposit,
    loadChildren: "./features/deposit/deposit.module#DepositModule",
    data: {
      breadcrumb: TRANSLATE("breadcrumb.deposit.root"),
    },
    canActivate: [AuthGuardService],
  },
  {
    path: AppRoutesEnum.organizationalUnit,
    loadChildren: "./features/organizational-unit/organizational-unit.module#OrganizationalUnitModule",
    data: {
      breadcrumb: TRANSLATE("breadcrumb.organizationalUnit.root"),
    },
    canActivate: [AuthGuardService],
  },
  {
    path: AppRoutesEnum.login,
    component: LoginPresentational,
  },
  {
    path: AppRoutesEnum.root,
    redirectTo: AppRoutesEnum.access,
    pathMatch: "full",
  },
  {
    path: AppRoutesEnum.index,
    redirectTo: AppRoutesEnum.access,
    pathMatch: "full",
  },
  {
    path: AppRoutesEnum.home,
    redirectTo: AppRoutesEnum.access,
    pathMatch: "full",
  },
  {
    path: "**",
    component: PageNotFoundPresentational,
  },
];

@NgModule({
  imports: [RouterModule.forRoot(routes)],
  exports: [RouterModule],
})
export class AppRoutingModule {
}
