import {ChangeDetectionStrategy, Component, Input, Output, ViewEncapsulation} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ThemeEnum} from "@app/shared/enums/theme.enum";
import {BehaviorSubject, Observable} from "rxjs";
import {StringUtil, ObservableUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-theme-selector",
  templateUrl: "./theme-selector.presentational.html",
  styleUrls: ["./theme-selector.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
  encapsulation: ViewEncapsulation.None,
})
export class ThemeSelectorPresentational extends SharedAbstractPresentational {
  listThemes: ThemeEnum[] = [ThemeEnum.dlcm, ThemeEnum.yareta];

  private readonly _themeBS: BehaviorSubject<ThemeEnum | undefined> = new BehaviorSubject<ThemeEnum | undefined>(undefined);

  get stringUtil(): typeof StringUtil {
    return StringUtil;
  }

  @Input()
  theme: ThemeEnum;

  @Output("themeChange")
  readonly themeObs: Observable<ThemeEnum | undefined> = ObservableUtil.asObservable(this._themeBS);

  constructor() {
    super();
  }

  changeTheme(theme: ThemeEnum): void {
    this._themeBS.next(theme);
  }
}
