import {ChangeDetectionStrategy, Component, Input} from "@angular/core";
import {DepositDataFile} from "@app/features/deposit/models/deposit-data-file.model";
import {UploadFileStatus} from "@app/features/deposit/models/upload-file-status.model";
import {DepositDataFileAction} from "@app/features/deposit/stores/data-file/deposit-data-file.action";
import {DepositAction} from "@app/features/deposit/stores/deposit.action";
import {SharedAbstractContainer} from "@app/shared/components/containers/shared-abstract/shared-abstract.container";
import {LocalStateEnum} from "../../../../../shared/enums/local-state.enum";
import {LocalStateModel} from "@app/shared/models/localStateModel";
import {Select, Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {StoreUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-file",
  templateUrl: "./deposit-file.container.html",
  styleUrls: ["./deposit-file.container.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFileContainer extends SharedAbstractContainer {
  @Select((state: LocalStateModel) => state[LocalStateEnum.deposit][LocalStateEnum.deposit_dataFile].list) listDataFileObs: Observable<DepositDataFile[]>;
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state[LocalStateEnum.deposit][LocalStateEnum.deposit_dataFile])) isLoadingDataFileObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.deposit].uploadStatus) uploadStatusObs: Observable<UploadFileStatus>;

  @Input()
  parentResId: string;

  @Input()
  readonly: boolean = false;

  constructor(protected store: Store) {
    super();
  }

  uploadFile(file: File): void {
    this.store.dispatch(new DepositAction.SendDataFile(this.parentResId, file));
  }

  delete($event: DepositDataFile): void {
    this.store.dispatch(new DepositDataFileAction.Delete(this.parentResId, $event));
  }

  resume($event: DepositDataFile): void {
    this.store.dispatch(new DepositDataFileAction.ResumeDataFile(this.parentResId, $event));
  }

  refresh(): void {
    this.store.dispatch(new DepositDataFileAction.GetAll(this.parentResId));
  }
}
