import {ChangeDetectionStrategy, Component, EventEmitter, Input, OnInit, Output} from "@angular/core";
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {UploadFileStatus} from "@app/features/deposit/models/upload-file-status.model";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {BehaviorSubject, Observable} from "rxjs";
import {ObservableUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-file-upload",
  templateUrl: "./deposit-file-upload.presentational.html",
  styleUrls: ["./deposit-file-upload.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFileUploadPresentational extends SharedAbstractPresentational implements OnInit {
  @Input()
  uploadStatus: UploadFileStatus;

  private readonly _fileUploadBS: BehaviorSubject<File | undefined> = new BehaviorSubject<File | undefined>(undefined);

  @Output("fileUploadChange")
  readonly fileUploadObs: Observable<File | undefined> = ObservableUtil.asObservable(this._fileUploadBS);

  form: FormGroup;
  readonly file: string = "file";

  constructor(protected fb: FormBuilder) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.form = this.fb.group({
      [this.file]: ["", Validators.required],
    });
  }

  onFileChange(event: Event & any): void {
    if (event.target.files.length > 0) {
      const file: File = event.target.files[0];
      this.form.get(this.file).setValue(file);
    }
  }

  onSubmit(): void {
    this._fileUploadBS.next(this.form.get(this.file).value as File);
  }
}
