import {ChangeDetectionStrategy, ChangeDetectorRef, Component, Input} from "@angular/core";
import {FormBuilder, Validators} from "@angular/forms";
import {
  Deposits,
  Languages,
  Licenses,
  OrganizationalUnits,
  PreservationPolicies,
  SubmissionPolicies,
} from "@app/generated-api";
import {SharedAbstractFormPresentational} from "@app/shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {BaseFormDefinition} from "@app/shared/models/base-form-definition.model";
import AccessEnum = Deposits.AccessEnum;
import {DateUtil, PropertyName, SolidifyValidator} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-form",
  templateUrl: "./deposit-form.presentational.html",
  styleUrls: ["../../../../../shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFormPresentational extends SharedAbstractFormPresentational<Deposits> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  get accessModelEnum(): typeof AccessEnum {
    return AccessEnum;
  }

  @Input()
  languages: Languages[];

  @Input()
  licenses: Licenses[];

  @Input()
  submissionPolicies: SubmissionPolicies[];

  @Input()
  preservationPolicies: PreservationPolicies[];

  @Input()
  organizationalUnits: OrganizationalUnits[];

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              private readonly _fb: FormBuilder) {
    super(_changeDetectorRef);
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.organizationalUnitId]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.title]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.languageId]: [""],
      [this.formDefinition.publicationDate]: [""],
      [this.formDefinition.collectionBegin]: [""],
      [this.formDefinition.collectionEnd]: [""],
      [this.formDefinition.author]: [""],
      [this.formDefinition.accessLevel]: [AccessEnum.PUBLIC],
      [this.formDefinition.hasEmbargo]: [false],
      [this.formDefinition.licenseId]: [""],
      [this.formDefinition.submissionPolicyId]: [""],
      [this.formDefinition.preservationPolicyId]: [""],
    });
  }

  protected bindFormTo(deposit: Deposits): void {
    this.form = this._fb.group({
      [this.formDefinition.organizationalUnitId]: [deposit.organizationalUnitId, [Validators.required, SolidifyValidator]],
      [this.formDefinition.title]: [deposit.title, [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: [deposit.description, [Validators.required, SolidifyValidator]],
      [this.formDefinition.languageId]: [deposit.languageId],
      [this.formDefinition.publicationDate]: [deposit.publicationDate],
      [this.formDefinition.collectionBegin]: [DateUtil.convertOffsetDateTimeIso8601ToDate(deposit.collectionBegin)],
      [this.formDefinition.collectionEnd]: [DateUtil.convertOffsetDateTimeIso8601ToDate(deposit.collectionEnd)],
      [this.formDefinition.author]: [""],
      [this.formDefinition.accessLevel]: [deposit.access],
      [this.formDefinition.hasEmbargo]: [deposit.hasEmbargo],
      [this.formDefinition.licenseId]: [deposit.licenseId],
      [this.formDefinition.submissionPolicyId]: [deposit.submissionPolicyId],
      [this.formDefinition.preservationPolicyId]: [deposit.preservationPolicyId],
    });
  }

  protected treatmentBeforeSubmit(deposit: Deposits): Deposits {
    deposit.publicationDate = DateUtil.convertToLocalDateDateSimple(deposit.publicationDate);
    deposit.collectionBegin = DateUtil.convertToOffsetDateTimeIso8601(deposit.collectionBegin);
    deposit.collectionEnd = DateUtil.convertToOffsetDateTimeIso8601(deposit.collectionEnd);
    return deposit;
  }

}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() organizationalUnitId: string;
  @PropertyName() title: string;
  @PropertyName() description: string;
  @PropertyName() languageId: string;
  @PropertyName() publicationDate: string;
  @PropertyName() collectionBegin: string;
  @PropertyName() collectionEnd: string;
  @PropertyName() author: string;
  @PropertyName() accessLevel: string;
  @PropertyName() hasEmbargo: string;
  @PropertyName() licenseId: string;
  @PropertyName() submissionPolicyId: string;
  @PropertyName() preservationPolicyId: string;
}
