import {ChangeDetectionStrategy, Component, OnInit, ViewChild} from "@angular/core";
import {depositActionNameSpace} from "@app/features/deposit/stores/deposit.action";
import {DepositStateModel} from "@app/features/deposit/stores/deposit.state";
import {Deposits, Languages, Licenses, PreservationPolicies, SubmissionPolicies} from "@app/generated-api";
import {OrganizationalUnitModule} from "@app/features/organizational-unit/organizational-unit.module";
import {SharedAbstractFormPresentational} from "@app/shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {SharedAbstractCreateRoutable} from "@app/shared/components/routables/shared-abstract-create/shared-abstract-create.routable";
import {LocalStateEnum} from "../../../../../shared/enums/local-state.enum";
import {LocalStateModel} from "@app/shared/models/localStateModel";
import {OrgUnitAction} from "@app/shared/stores/organizational-unit/organizational-unit.action";
import {Select, Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {FormState} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-create-routable",
  templateUrl: "./deposit-create.routable.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-create/shared-abstract-create.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositCreateRoutable extends SharedAbstractCreateRoutable<Deposits, DepositStateModel> implements OnInit {
  @Select((state: LocalStateModel) => state[LocalStateEnum.deposit].form) formObs: Observable<FormState>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.language].list) languagesObs: Observable<Languages[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.license].list) licensesObs: Observable<Licenses[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.submissionPolicy].list) submissionPoliciesObs: Observable<SubmissionPolicies[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.preservationPolicy].list) preservationPoliciesObs: Observable<PreservationPolicies[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.organizationalUnit].list) organizationalUnitsObs: Observable<OrganizationalUnitModule[]>;

  @ViewChild("formPresentational")
  readonly formPresentational: SharedAbstractFormPresentational<Deposits>;

  constructor(protected store: Store) {
    super(store, LocalStateEnum.deposit, depositActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.store.dispatch(new OrgUnitAction.GetAll());
  }
}
