import {ChangeDetectionStrategy, Component, OnInit} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {DepositDataFileAction} from "@app/features/deposit/stores/data-file/deposit-data-file.action";
import {depositActionNameSpace} from "@app/features/deposit/stores/deposit.action";
import {DepositStateModel} from "@app/features/deposit/stores/deposit.state";
import {Deposits, Languages, Licenses, PreservationPolicies, SubmissionPolicies} from "@app/generated-api";
import {OrganizationalUnitModule} from "@app/features/organizational-unit/organizational-unit.module";
import {LocalStateEnum} from "../../../../../shared/enums/local-state.enum";
import {LocalStateModel} from "@app/shared/models/localStateModel";
import {OrgUnitAction} from "@app/shared/stores/organizational-unit/organizational-unit.action";
import {SharedAbstractEditRoutable} from "@app/shared/components/routables/shared-abstract-edit/shared-abstract-edit.routable";
import {Select, Store} from "@ngxs/store";
import {Observable} from "rxjs";

@Component({
  selector: "dlcm-deposit-edit-routable",
  templateUrl: "./deposit-edit.routable.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-edit/shared-abstract-edit.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositEditRoutable extends SharedAbstractEditRoutable<Deposits, DepositStateModel> implements OnInit {
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.language].list) languagesObs: Observable<Languages[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.license].list) licensesObs: Observable<Licenses[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.submissionPolicy].list) submissionPoliciesObs: Observable<SubmissionPolicies[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.preservationPolicy].list) preservationPoliciesObs: Observable<PreservationPolicies[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.organizationalUnit].list) organizationalUnitsObs: Observable<OrganizationalUnitModule[]>;

  constructor(protected store: Store,
              protected route: ActivatedRoute) {
    super(store, route, LocalStateEnum.deposit, depositActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.store.dispatch(new OrgUnitAction.GetAll());
  }

  getSubResourceWithParentId(id: string): void {
    this.store.dispatch(new DepositDataFileAction.GetAll(id));
  }
}
