import {ChangeDetectionStrategy, Component} from "@angular/core";
import {depositActionNameSpace} from "@app/features/deposit/stores/deposit.action";
import {DepositStateModel} from "@app/features/deposit/stores/deposit.state";
import {Deposits} from "@app/generated-api";
import {SharedAbstractListRoutable} from "@app/shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {LocalStateEnum} from "../../../../../shared/enums/local-state.enum";
import {TranslateService} from "@ngx-translate/core";
import {Store} from "@ngxs/store";
import {TRANSLATE} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-list-routable",
  templateUrl: "../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositListRoutable extends SharedAbstractListRoutable<Deposits, DepositStateModel> {
  readonly KEY_CREATE_BUTTON: string = TRANSLATE("deposit.new");
  readonly KEY_REFRESH_BUTTON: string = TRANSLATE("deposit.refresh");

  constructor(protected store: Store, private translate: TranslateService) {
    super(store, LocalStateEnum.deposit, depositActionNameSpace);
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "title",
        header: TRANSLATE("deposit.table.header.title"),
        type: FieldTypeEnum.string,
        order: 0,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "publicationDate",
        header: TRANSLATE("deposit.table.header.publicationDate"),
        type: FieldTypeEnum.date,
        order: 0,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "creation.when" as any,
        header: TRANSLATE("deposit.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: 0,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: TRANSLATE("deposit.table.header.lastUpdate.when"),
        type: FieldTypeEnum.datetime,
        order: 0,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "status",
        header: TRANSLATE("deposit.table.header.status"),
        type: FieldTypeEnum.string,
        order: 0,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }
}
