import {DepositDataFile} from "@app/features/deposit/models/deposit-data-file.model";
import {DepositDataFileAction, depositDataFileActionNameSpace} from "@app/features/deposit/stores/data-file/deposit-data-file.action";
import {ApiActionEnum} from "@app/shared/enums/api-action.enum";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {PreIngestResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "../../../../shared/enums/local-state.enum";
import {Action, State, StateContext, Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {CompositionStateModel, defaultCompositionStateInitValue, CompositionState, ApiService, ResourceApiEnum, CompositionNameSpace,
  TRANSLATE,
  NotificationService} from "solidify-frontend";

export interface DepositDataFileStateModel extends CompositionStateModel<DepositDataFile> {
}

@State<DepositDataFileStateModel>({
  name: LocalStateEnum.deposit_dataFile,
  defaults: {
    ...defaultCompositionStateInitValue,
  },
})
export class DepositDataFileState extends CompositionState<DepositDataFile> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService
  ) {
    super(apiService, store, notificationService);
  }

  defineUrlResource(): ResourceApiEnum {
    return PreIngestResourceApiEnum.deposits;
  }

  defineCompositionNameSpace(): CompositionNameSpace {
    return depositDataFileActionNameSpace;
  }

  defineState(): LocalStateEnum {
    return LocalStateEnum.deposit_dataFile;
  }

  defineResourceName(): ApiResourceNameEnum {
    return ApiResourceNameEnum.DATAFILE;
  }

  @Action(DepositDataFileAction.ResumeDataFile)
  resume(ctx: StateContext<DepositDataFileState>, action: DepositDataFileAction.ResumeDataFile): Observable<any> {
    return this.apiService.post<void>(`${this.urlResource}/${action.parentId}/${ApiResourceNameEnum.DATAFILE}/${action.dataFile.resId}/${ApiActionEnum.RESUME}`)
      .pipe(
        tap(() => {
          // this.notificationService.showInformation(TRANSLATE("notification.deposit.file.resumed"), true);
          ctx.dispatch(new DepositDataFileAction.GetAll(action.parentId));
        }),
      );
  }
}
