import {ChangeDetectionStrategy, ChangeDetectorRef, Component, Input, OnChanges, SimpleChanges} from "@angular/core";
import {FormBuilder, Validators} from "@angular/forms";
import {OrganizationalUnits, PreservationPolicies, SubmissionPolicies} from "@app/generated-api";
import {SharedAbstractFormPresentational} from "@app/shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {LocalModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {BaseFormDefinition} from "@app/shared/models/base-form-definition.model";
import _ from "lodash";
import {DateUtil, PropertyName, SolidifyValidator} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-form",
  templateUrl: "./orgunit-form.presentational.html",
  styleUrls: ["../../../../../shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitFormPresentational extends SharedAbstractFormPresentational<OrganizationalUnits> implements OnChanges {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  @Input()
  selectedSubmissionPolicies: SubmissionPolicies[];

  @Input()
  listSubmissionPolicies: SubmissionPolicies[];

  @Input()
  listPreservationPolicies: PreservationPolicies[];

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              private readonly _fb: FormBuilder) {
    super(_changeDetectorRef);
  }

  ngOnChanges(changes: SimpleChanges): void {
    super.ngOnChanges(changes);

    if (this.form === undefined) {
      return;
    }

    if (changes.selectedSubmissionPolicies && changes.selectedSubmissionPolicies.currentValue.length !== 0) {
      const newValue = _.map(changes.selectedSubmissionPolicies.currentValue, LocalModelAttributeEnum.resId);
      this.form.get(this.formDefinition.submissionPolicies).setValue(newValue);
    }
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.openingDate]: [""],
      [this.formDefinition.closingDate]: [""],
      [this.formDefinition.isEmpty]: [false],
      [this.formDefinition.open]: [false],
      [this.formDefinition.submissionPolicies]: [],
    });
  }

  protected bindFormTo(organizationalUnit: OrganizationalUnits): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: [organizationalUnit.name, [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: [organizationalUnit.description, [Validators.required, SolidifyValidator]],
      [this.formDefinition.openingDate]: [organizationalUnit.openingDate],
      [this.formDefinition.closingDate]: [organizationalUnit.closingDate],
      [this.formDefinition.isEmpty]: [organizationalUnit.isEmpty],
      [this.formDefinition.open]: [organizationalUnit.open],
      [this.formDefinition.submissionPolicies]: [_.map(this.selectedSubmissionPolicies, LocalModelAttributeEnum.resId)],
    });
  }

  protected treatmentBeforeSubmit(organizationalUnit: OrganizationalUnits): OrganizationalUnits {
    organizationalUnit.openingDate = DateUtil.convertToLocalDateDateSimple(organizationalUnit.openingDate);
    organizationalUnit.closingDate = DateUtil.convertToLocalDateDateSimple(organizationalUnit.closingDate);
    return organizationalUnit;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() name: string;
  @PropertyName() description: string;
  @PropertyName() isEmpty: string;
  @PropertyName() closingDate: string;
  @PropertyName() openingDate: string;
  @PropertyName() submissionPolicies: string;
  @PropertyName() open: string;
}
