import {ChangeDetectionStrategy, Component, Input, OnInit, Output} from "@angular/core";
import {FormBuilder, FormGroup} from "@angular/forms";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {BehaviorSubject, Observable} from "rxjs";
import {distinctUntilChanged} from "rxjs/internal/operators/distinctUntilChanged";
import {tap} from "rxjs/internal/operators/tap";
import {debounceTime} from "rxjs/operators";
import {ObservableUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-search",
  templateUrl: "./shared-search.presentational.html",
  styleUrls: ["./shared-search.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedSearchPresentational extends SharedAbstractPresentational implements OnInit {
  searchForm: FormGroup;
  search: string = "search";

  @Input()
  value: string;

  private readonly _valueBS: BehaviorSubject<string | undefined> = new BehaviorSubject<string | undefined>(undefined);

  @Output("valueChange")
  readonly valueObs: Observable<string | undefined> = ObservableUtil.asObservable(this._valueBS);

  private _debounceTime: number = 600;

  constructor(private fb: FormBuilder) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.searchForm = this.fb.group({
      [this.search]: this.value === null || this.value === undefined ? "" : this.value,
    });

    this.subscribe(
      this.searchForm.get(this.search).valueChanges
        .pipe(
          debounceTime(this._debounceTime),
          distinctUntilChanged(),
          tap((value: string) => {
            this._valueBS.next(value.trim().toLocaleLowerCase());
          }),
        ),
    );
  }

}
