import {OnInit} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {SharedAbstractRoutable} from "@app/shared/components/routables/shared-abstract/shared-abstract.routable";
import {LocalStateEnum} from "../../../enums/local-state.enum";
import {Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {ResourceStateModel, ResourceNameSpace, StoreUtil, ResourceActionHelper, ModelFormControlEvent} from "solidify-frontend";

export abstract class SharedAbstractEditRoutable<T, U extends ResourceStateModel<T>> extends SharedAbstractRoutable implements OnInit {
  isLoadingObs: Observable<boolean>;
  currentObs: Observable<T>;

  private resId: string;

  protected constructor(protected store: Store,
                        protected route: ActivatedRoute,
                        private state: LocalStateEnum,
                        private resourceNameSpace: ResourceNameSpace) {
    super();
    this.isLoadingObs = this.store.select(s => StoreUtil.isLoadingState(s[state] as U));
    this.currentObs = this.store.select(s => (s[state] as U).current);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.store.dispatch(ResourceActionHelper.loadResource(this.resourceNameSpace));
    this.getCurrentModel();
  }

  private getCurrentModel(): void {
    this.resId = this.route.snapshot.paramMap.get("id");
    this.getById(this.resId);
  }

  private getById(id: string): void {
    this.store.dispatch(ResourceActionHelper.getById(this.resourceNameSpace, id));
    this.getSubResourceWithParentId(id);
  }

  abstract getSubResourceWithParentId(id: string): void;

  getResId(): string {
    return this.resId;
  }

  update(model: ModelFormControlEvent<T>): void {
    this.store.dispatch(ResourceActionHelper.update<T>(this.resourceNameSpace, model));
  }
}
