import {OnInit} from "@angular/core";
import {SharedAbstractRoutable} from "@app/shared/components/routables/shared-abstract/shared-abstract.routable";
import {LocalModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {DataTableColumns} from "@app/shared/models/data-table-columns.model";
import {StoreRouteLocalUtil} from "@app/shared/utils/store-route-local.util";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {QueryParameters, ResourceActionHelper, ResourceNameSpace, ResourceStateModel, StoreUtil} from "solidify-frontend";
import {environment} from "../../../../../environments/environment";
import {LocalStateEnum} from "../../../enums/local-state.enum";

export abstract class SharedAbstractListRoutable<T, U extends ResourceStateModel<T>> extends SharedAbstractRoutable implements OnInit {
  isLoadingObs: Observable<boolean>;
  listObs: Observable<T[]>;
  queryParametersObs: Observable<QueryParameters>;

  columns: DataTableColumns<T>[];

  abstract readonly KEY_CREATE_BUTTON: string;
  abstract readonly KEY_REFRESH_BUTTON: string;

  protected constructor(protected store: Store, private state: LocalStateEnum, private resourceNameSpace: ResourceNameSpace) {
    super();
    this.isLoadingObs = this.store.select(s => StoreUtil.isLoadingState(s[state] as U));
    this.listObs = this.store.select(s => (s[state] as U).list);
    this.queryParametersObs = this.store.select(s => (s[state] as U).queryParameters);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.defineColumns();
    this.getAll(new QueryParameters(environment.defaultPageSizeOption));
  }

  abstract defineColumns(): void;

  create(): void {
    this.store.dispatch(new Navigate([StoreRouteLocalUtil.getCreateRoute(this.state)]));
  }

  getAll(queryParameters?: QueryParameters): void {
    this.store.dispatch(ResourceActionHelper.getAll(this.resourceNameSpace, queryParameters));
  }

  showDetail(model: T): void {
    this.store.dispatch(new Navigate([StoreRouteLocalUtil.getDetailRoute(this.state), model[LocalModelAttributeEnum.resId]]));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this.store.dispatch(ResourceActionHelper.changeQueryParameters(this.resourceNameSpace, queryParameters));
  }
}
