import {HttpClientTestingModule} from "@angular/common/http/testing";
import {TestBed} from "@angular/core/testing";
import {MatSnackBar} from "@angular/material";
import {Router} from "@angular/router";
import {AuthGuardService} from "@app/shared/guards/auth-guard.service";
import {LoginFail, LoginSuccess} from "@app/stores/app.action";
import {AppState} from "@app/stores/app.state";
import {TranslateService} from "@ngx-translate/core";
import {NgxsModule, Store} from "@ngxs/store";
import {ENVIRONMENT, SNACK_BAR} from "solidify-frontend";
import {environment} from "../../../environments/environment";


describe("AuthGuardService", () => {
  const router = {
    navigate: jasmine.createSpy("navigate")
  };
  const mockTranslateService = jasmine.createSpyObj("TranslateService", [ "getTranslation", "setDefaultLang", "use" ]);

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule, NgxsModule.forRoot([AppState])],
      providers: [
        {
          provide: AuthGuardService,
          useClass: AuthGuardService
        },
        {
          provide: TranslateService,
          useValue: mockTranslateService
        },
        {
          provide: Router,
          useValue: router
        },
        {
          provide: ENVIRONMENT,
          useValue: environment,
        },
        {
          provide: SNACK_BAR,
          useClass: MatSnackBar,
        },
      ]
    });
  });

  it("should be created", () => {
    const service: AuthGuardService = TestBed.get(AuthGuardService);
    expect(service).toBeTruthy();
  });


  it("be able to hit route when user is logged in", async () => {
    const service: AuthGuardService = TestBed.get(AuthGuardService);
    const store: Store = TestBed.get(Store);

    await store.dispatch(new LoginSuccess(null)).toPromise();
    const state = store.selectSnapshot(AppState);

    expect(state.isLoggedIn).toBe(true);
    expect(service.canActivate()).toBe(true);
  });

  it("not be able to hit route when user is not logged in", async () => {
    const service: AuthGuardService = TestBed.get(AuthGuardService);
    const store: Store = TestBed.get(Store);

    await store.dispatch(new LoginFail(null)).toPromise();
    const state = store.selectSnapshot(AppState);

    expect(state.isLoggedIn).toBe(false);
    expect(service.canActivate()).toBe(false);
  });

});
