import {Injectable} from "@angular/core";
import {CanActivate, Router} from "@angular/router";
import {AppStateModel} from "@app/stores/app.state";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {LocalStateEnum} from "../enums/local-state.enum";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";

@Injectable({
  providedIn: "root"
})
export class AuthGuardService implements CanActivate {
  constructor(public router: Router, public store: Store) {
  }

  canActivate(): boolean {
    const isLoggedIn: boolean = this.store.selectSnapshot<boolean>(state => (state[LocalStateEnum.application] as AppStateModel).isLoggedIn);
    if (isLoggedIn === false) {
      this.store.dispatch(new Navigate([RoutesEnum.index]));
    }
    return isLoggedIn;
  }

}
