import {OrganizationalUnits, SubmissionPolicies} from "@app/generated-api";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "../../../enums/local-state.enum";
import {orgUnitSubmissionPolicyActionNameSpace} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.action";
import {
  ApiService,
  CollectionTyped,
  isNullOrUndefined,
  MapUtil,
  QueryParameters,
  QueryParametersUtil,
  ResourceStateModel,
  StoreUtil,
  StringUtil,
  ResourceState,
  ResourceNameSpace,
  ResourceApiEnum,
  defaultResourceStateInitValue,
  Relation2TiersStateModel,
  defaultRelation2TiersStateInitValue,
  Relation2TiersState,
  Relation2TiersNameSpace,
  NotificationService,
} from "solidify-frontend";

import {Actions, State, Store} from "@ngxs/store";

export interface OrganizationalUnitSubmissionPolicyStateModel extends Relation2TiersStateModel<SubmissionPolicies> {
}

@State<OrganizationalUnitSubmissionPolicyStateModel>({
  name: LocalStateEnum.organizationalUnit_submissionPolicy,
  defaults: {
    ...defaultRelation2TiersStateInitValue,
  },
})
export class OrganizationalUnitSubmissionPolicyState extends Relation2TiersState<OrganizationalUnits> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$);
  }

  defineUrlResource(): ResourceApiEnum {
    return AdminResourceApiEnum.organizationalUnits;
  }

  defineState(): LocalStateEnum {
    return LocalStateEnum.organizationalUnit;
  }

  defineRelation2TiersNameSpace(): Relation2TiersNameSpace {
    return orgUnitSubmissionPolicyActionNameSpace;
  }

  defineResourceName(): ApiResourceNameEnum {
    return ApiResourceNameEnum.SUB_POLICY;
  }
}
