import {Licenses} from "@app/generated-api";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {isNullOrUndefined} from "util";
import Metadata = ArchiveMetadata.Metadata;
import {StringUtil} from "solidify-frontend";

export class MetadataUtil {
  static readonly aipDispositionApproval: string = "aip-disposition-approval";
  static readonly aipOrganizationalUnit: string = "aip-organizational-unit";
  static readonly aipRetention: string = "aip-retention";
  static readonly aipRetentionEnd: string = "aip-retention-end";
  static readonly dataciteXml: string = "datacite.xml";
  static readonly aipContainer: string = "aip-container";

  static readonly datacite: string = "datacite";
  static readonly dataciteResource: string = "datacite:resource";
  static readonly xmlnsDatacite: string = "xmlns:datacite";
  static readonly dataciteDates: string = "datacite:dates";
  static readonly dataciteDate: string = "datacite:date";
  static readonly dataciteTitles: string = "datacite:titles";
  static readonly dataciteTitle: string = "datacite:title";
  static readonly dataciteResourceType: string = "datacite:resourceType";
  static readonly resourceTypeGeneral: string = "resourceTypeGeneral";
  static readonly content: string = "content";
  static readonly dataciteRelatedIdentifiers: string = "datacite:relatedIdentifiers";
  static readonly dataciteRelatedIdentifier: string = "datacite:relatedIdentifier";
  static readonly relationType: string = "relationType";
  static readonly relatedIdentifierType: string = "relatedIdentifierType";
  static readonly xmlnsDlcm: string = "xmlns:dlcm";
  static readonly xmlnsMets: string = "xmlns:mets";
  static readonly dataciteFormats: string = "datacite:formats";
  static readonly dataciteFormat: string = "datacite:format";
  static readonly dataciteIdentifier: string = "datacite:identifier";
  static readonly identifierType: string = "identifierType";
  static readonly dataciteRightsList: string = "datacite:rightsList";
  static readonly dataciteRights: string = "datacite:rights";
  static readonly rightsURI: string = "rightsURI";
  static readonly xmlnsXlink: string = "xmlns:xlink";
  static readonly datacitePublicationYear: string = "datacite:publicationYear";
  static readonly datacitePublisher: string = "datacite:publisher";
  static readonly dataciteDescriptions: string = "datacite:descriptions";
  static readonly dataciteDescription: string = "datacite:description";
  static readonly descriptionType: string = "descriptionType";
  static readonly dataciteCreators: string = "datacite:creators";
  static readonly dataciteCreator: string = "datacite:creator";
  static readonly dataciteCreatorName: string = "datacite:creatorName";
  static readonly xmlnsFits: string = "xmlns:fits";
  static readonly xmlnsPremis: string = "xmlns:premis";
  static readonly aipUnit: string = "aip-unit";
  static readonly creation: string = "creation";

  static readonly rightsUtiForAccessLevelFinal: string = "https://www.dlcm.ch/access-level/final";
  static readonly rightsUtiForAccessLevel: string = "https://www.dlcm.ch";

  static getTitle(metadata: Metadata): string {
    if (isNullOrUndefined(metadata) || isNullOrUndefined(metadata[MetadataUtil.datacite])) {
      return StringUtil.stringEmpty;
    }
    return metadata[MetadataUtil.datacite][MetadataUtil.dataciteResource][MetadataUtil.dataciteTitles][MetadataUtil.dataciteTitle][0];
  }

  static getOrganizationalUnitResId(metadata: Metadata): string {
    return metadata[MetadataUtil.aipOrganizationalUnit];
  }

  static getPublicationYear(metadata: Metadata): number {
    if (isNullOrUndefined(metadata) || isNullOrUndefined(metadata[MetadataUtil.datacite])) {
      return 0;
    }
    return metadata[MetadataUtil.datacite][MetadataUtil.dataciteResource][MetadataUtil.datacitePublicationYear];
  }

  static getAccessLevel(metadata: Metadata): string {
    if (isNullOrUndefined(metadata) || isNullOrUndefined(metadata[MetadataUtil.datacite])) {
      return StringUtil.stringEmpty;
    }
    const listRight = MetadataUtil.getListRights(metadata);
    let accessLevel = listRight.find(r => r.rightsURI === MetadataUtil.rightsUtiForAccessLevelFinal);
    if (accessLevel !== null && accessLevel !== undefined) {
      return accessLevel.content;
    }
    accessLevel = listRight.find(r => r.rightsURI === MetadataUtil.rightsUtiForAccessLevel);
    return accessLevel === null || accessLevel === undefined ? null : accessLevel.content;
  }

  static getLicences(metadata: Metadata): Licenses[] {
    if (isNullOrUndefined(metadata) || isNullOrUndefined(metadata[MetadataUtil.datacite])) {
      return [];
    }
    const listRight = MetadataUtil.getListRights(metadata);
    const licenses = listRight.filter(r => r.rightsURI !== MetadataUtil.rightsUtiForAccessLevelFinal && r.rightsURI !== MetadataUtil.rightsUtiForAccessLevel);

    const listLicences: Licenses[] = [];
    licenses.forEach(l => listLicences.push({title: l.content, url: l.rightsURI} as Licenses));
    return listLicences;
  }

  private static getListRights(metadata: Metadata): [{ rightsURI: string, content: string }] {
    return metadata[MetadataUtil.datacite][MetadataUtil.dataciteResource][MetadataUtil.dataciteRightsList][MetadataUtil.dataciteRights];
  }
}
