import {LanguagesEnum} from "@app/shared/enums/languages.enum";
import {SessionStorageEnum} from "@app/shared/enums/session-storage.enum";
import {environment} from "../../../environments/environment";

export class UserPreferencesUtil {
  static getPreferredLanguage(): LanguagesEnum {
    const savedLanguage = sessionStorage.getItem(SessionStorageEnum.language);
    if (savedLanguage === null) {
      return this.determineDefaultLanguageAndStoreIntoSessionStorage();
    }
    return savedLanguage as LanguagesEnum;
  }

  private static determineDefaultLanguageAndStoreIntoSessionStorage(): LanguagesEnum {
    const defaultLanguageDetermined = UserPreferencesUtil.determineDefaultLanguage();
    sessionStorage.setItem(SessionStorageEnum.language, defaultLanguageDetermined);
    return defaultLanguageDetermined;
  }

  private static determineDefaultLanguage(): LanguagesEnum {
    const languageFromLocalBrowser = UserPreferencesUtil.findCorrespondingLanguageToLocal();
    if (languageFromLocalBrowser === null) {
      return environment.defaultLanguage;
    }
    return languageFromLocalBrowser;
  }

  private static findCorrespondingLanguageToLocal(): LanguagesEnum {
    const browserLanguage: string = navigator.language;

    if (browserLanguage.startsWith(LanguagesEnum.en)) {
      return LanguagesEnum.en;
    }
    if (browserLanguage.startsWith(LanguagesEnum.fr)) {
      return LanguagesEnum.fr;
    }
    if (browserLanguage.startsWith(LanguagesEnum.de)) {
      return LanguagesEnum.de;
    }

    return null;
  }
}
